!   C06PUF Example Program Text
!   Mark 23 Release. NAG Copyright 2011.

    MODULE c06pufe_mod

!      C06PUF Example Program Module:
!             Parameters and User-defined Routines

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER                  :: nin = 5, nout = 6
    CONTAINS
!dec$ attributes offload : mic :: readx
       SUBROUTINE readx(nin,x,n1,n2)
!         Read 2-dimensional complex data

!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Scalar Arguments ..
          INTEGER, INTENT (IN)                :: n1, n2, nin
!         .. Array Arguments ..
          COMPLEX (KIND=nag_wp), INTENT (OUT) :: x(n1,n2)
!         .. Local Scalars ..
          INTEGER                             :: i, j
!         .. Executable Statements ..
          DO i = 1, n1
             READ (nin,*) (x(i,j),j=1,n2)
          END DO
          RETURN
!dec$ attributes offload : mic :: readx
       END SUBROUTINE readx

!dec$ attributes offload : mic :: writx
       SUBROUTINE writx(nout,x,n1,n2)
!         Print 2-dimensional complex data

!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Scalar Arguments ..
          INTEGER, INTENT (IN)                :: n1, n2, nout
!         .. Array Arguments ..
          COMPLEX (KIND=nag_wp), INTENT (IN)  :: x(n1,n2)
!         .. Local Scalars ..
          INTEGER                             :: i, j
!         .. Intrinsic Functions ..
          INTRINSIC                              aimag, real
!         .. Executable Statements ..
          DO i = 1, n1
             WRITE (nout,*)
             WRITE (nout,99999) 'Real ', (real(x(i,j)),j=1,n2)
             WRITE (nout,99999) 'Imag ', (aimag(x(i,j)),j=1,n2)
          END DO
          RETURN

99999     FORMAT (1X,A,7F10.3/(6X,7F10.3))
!dec$ attributes offload : mic :: writx
       END SUBROUTINE writx
    END MODULE c06pufe_mod

    PROGRAM c06pufe

!      C06PUF Example Main Program

!      .. Use Statements ..
       USE nag_library, ONLY : c06puf, nag_wp
       USE c06pufe_mod, ONLY : nin, nout, readx, writx
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Local Scalars ..
       INTEGER                             :: ieof, ifail, m, n
!      .. Local Arrays ..
       COMPLEX (KIND=nag_wp), ALLOCATABLE  :: work(:), x(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'C06PUF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
LOOP:  DO
          READ (nin,*,IOSTAT=ieof) m, n
          IF (ieof<0) EXIT LOOP
          ALLOCATE (work(m*n+n+m+30),x(m*n))
          CALL readx(nin,x,m,n)
          WRITE (nout,*)
          WRITE (nout,*) 'Original data values'
          CALL writx(nout,x,m,n)

!         ifail: behaviour on error exit   
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
!         -- Compute transform
          CALL c06puf('F',m,n,x,work,ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Components of discrete Fourier transform'
          CALL writx(nout,x,m,n)

!         -- Compute inverse transform
          CALL c06puf('B',m,n,x,work,ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Original sequence as restored by inverse transform'
          CALL writx(nout,x,m,n)
          DEALLOCATE (x,work)

       END DO LOOP

    END PROGRAM c06pufe
