    PROGRAM f07bbfe

!      F07BBF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dgbsvx, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: rcond
       INTEGER                         :: i, ifail, info, j, k, kl, ku, ldab,  &
                                          ldafb, ldb, ldx, n, nrhs
       CHARACTER (1)                   :: equed
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: ab(:,:), afb(:,:), b(:,:), berr(:),  &
                                          c(:), ferr(:), r(:), work(:), x(:,:)
       INTEGER, ALLOCATABLE            :: ipiv(:), iwork(:)
!      .. Intrinsic Functions ..
       INTRINSIC                          max, min
!      .. Executable Statements ..
       WRITE (nout,*) 'F07BBF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs, kl, ku
       ldb = n
       ldx = n
       ldab = kl + ku + 1
       ldafb = ldab + kl
       ALLOCATE (ab(ldab,n),afb(ldafb,n),b(ldb,nrhs),berr(nrhs),c(n), &
          ferr(nrhs),r(n),work(3*n),x(ldx,nrhs),ipiv(n),iwork(n))

!      Read the band matrix A and B from data file

       k = ku + 1
       READ (nin,*) ((ab(k+i-j,j),j=max(i-kl,1),min(i+ku,n)),i=1,n)
       READ (nin,*) (b(i,1:nrhs),i=1,n)

!      Solve the equations AX = B for X
!      The NAG name equivalent of dgbsvx is f07bbf
       CALL dgbsvx('Equilibration','No transpose',n,kl,ku,nrhs,ab,ldab,afb, &
          ldafb,ipiv,equed,r,c,b,ldb,x,ldx,rcond,ferr,berr,work,iwork,info)

       IF ((info==0) .OR. (info==n+1)) THEN

!         Print solution, error bounds, condition number, the form
!         of equilibration and the pivot growth factor

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04caf('General',' ',n,nrhs,x,ldx,'Solution(s)',ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Backward errors (machine-dependent)'
          WRITE (nout,99999) berr(1:nrhs)
          WRITE (nout,*)
          WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
          WRITE (nout,99999) ferr(1:nrhs)
          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of reciprocal condition number'
          WRITE (nout,99999) rcond
          WRITE (nout,*)
          IF (equed=='N') THEN
             WRITE (nout,*) 'A has not been equilibrated'
          ELSE IF (equed=='R') THEN
             WRITE (nout,*) 'A has been row scaled as diag(R)*A'
          ELSE IF (equed=='C') THEN
             WRITE (nout,*) 'A has been column scaled as A*diag(C)'
          ELSE IF (equed=='B') THEN
             WRITE (nout,*) &
                'A has been row and column scaled as diag(R)*A*diag(C)'
          END IF
          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of reciprocal pivot growth factor'
          WRITE (nout,99999) work(1)

          IF (info==n+1) THEN
             WRITE (nout,*)
             WRITE (nout,*) 'The matrix A is singular to working precision'
          END IF
       ELSE
          WRITE (nout,99998) 'The (', info, ',', info, ')', &
             ' element of the factor U is zero'
       END IF

99999  FORMAT ((3X,1P,7E11.1))
99998  FORMAT (1X,A,I3,A,I3,A,A)
    END PROGRAM f07bbfe
