    PROGRAM f07pnfe

!      F07PNF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp, x04ddf, zhpsv
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
       CHARACTER (1), PARAMETER        :: uplo = 'U'
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, info, j, n
!      .. Local Arrays ..
       COMPLEX (KIND=nag_wp), ALLOCATABLE :: ap(:), b(:)
       INTEGER, ALLOCATABLE            :: ipiv(:)
       CHARACTER (1)                   :: clabs(1), rlabs(1)
!      .. Executable Statements ..
       WRITE (nout,*) 'F07PNF Example Program Results'
       WRITE (nout,*)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n

       ALLOCATE (ap((n*(n+1))/2),b(n),ipiv(n))

!      Read the upper or lower triangular part of the matrix A from
!      data file

       IF (uplo=='U') THEN
          READ (nin,*) ((ap(i+(j*(j-1))/2),j=i,n),i=1,n)
       ELSE IF (uplo=='L') THEN
          READ (nin,*) ((ap(i+((2*n-j)*(j-1))/2),j=1,i),i=1,n)
       END IF

!      Read b from data file

       READ (nin,*) b(1:n)

!      Solve the equations Ax = b for x
!      The NAG name equivalent of zhpsv is f07pnf
       CALL zhpsv(uplo,n,1,ap,ipiv,b,n,info)

       IF (info==0) THEN

!         Print solution

          WRITE (nout,*) 'Solution'
          WRITE (nout,99999) b(1:n)

!         Print details of factorization

          WRITE (nout,*)
          FLUSH (nout)

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04ddf(uplo,'Non-unit diagonal',n,ap,'Bracketed','F7.4', &
             'Details of factorization','Integer',rlabs,'Integer',clabs,80,0, &
             ifail)

!         Print pivot indices

          WRITE (nout,*)
          WRITE (nout,*) 'Pivot indices'
          WRITE (nout,99998) ipiv(1:n)

       ELSE
          WRITE (nout,99997) 'The diagonal block ', info, ' of D is zero'
       END IF

99999  FORMAT ((3X,4(' (',F7.4,',',F7.4,')':)))
99998  FORMAT (1X,7I11)
99997  FORMAT (1X,A,I3,A)
    END PROGRAM f07pnfe
