    PROGRAM f07qvfe

!      F07QVF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp, x04dbf, zsprfs, zsptrf, zsptrs
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       INTEGER                         :: aplen, i, ifail, info, j, ldb, ldx,  &
                                          n, nrhs
       CHARACTER (1)                   :: uplo
!      .. Local Arrays ..
       COMPLEX (KIND=nag_wp), ALLOCATABLE :: afp(:), ap(:), b(:,:), work(:),   &
                                             x(:,:)
       REAL (KIND=nag_wp), ALLOCATABLE :: berr(:), ferr(:), rwork(:)
       INTEGER, ALLOCATABLE            :: ipiv(:)
       CHARACTER (1)                   :: clabs(1), rlabs(1)
!      .. Executable Statements ..
       WRITE (nout,*) 'F07QVF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs
       ldb = n
       ldx = n
       aplen = n*(n+1)/2
       ALLOCATE (afp(aplen),ap(aplen),b(ldb,nrhs),work(2*n),x(ldx,n), &
          berr(nrhs),ferr(nrhs),rwork(n),ipiv(n))

!      Read A and B from data file, and copy A to AFP and B to X

       READ (nin,*) uplo
       IF (uplo=='U') THEN
          READ (nin,*) ((ap(i+j*(j-1)/2),j=i,n),i=1,n)
       ELSE IF (uplo=='L') THEN
          READ (nin,*) ((ap(i+(2*n-j)*(j-1)/2),j=1,i),i=1,n)
       END IF
       READ (nin,*) (b(i,1:nrhs),i=1,n)

       afp(1:aplen) = ap(1:aplen)
       x(1:n,1:nrhs) = b(1:n,1:nrhs)

!      Factorize A in the array AFP
!      The NAG name equivalent of zsptrf is f07qrf
       CALL zsptrf(uplo,n,afp,ipiv,info)

       WRITE (nout,*)
       FLUSH (nout)
       IF (info==0) THEN

!         Compute solution in the array X
!         The NAG name equivalent of zsptrs is f07qsf
          CALL zsptrs(uplo,n,nrhs,afp,ipiv,x,ldx,info)

!         Improve solution, and compute backward errors and
!         estimated bounds on the forward errors

!         The NAG name equivalent of zsprfs is f07qvf
          CALL zsprfs(uplo,n,nrhs,ap,afp,ipiv,b,ldb,x,ldx,ferr,berr,work, &
             rwork,info)

!         Print solution

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04dbf('General',' ',n,nrhs,x,ldx,'Bracketed','F7.4', &
             'Solution(s)','Integer',rlabs,'Integer',clabs,80,0,ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Backward errors (machine-dependent)'
          WRITE (nout,99999) berr(1:nrhs)
          WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
          WRITE (nout,99999) ferr(1:nrhs)
       ELSE
          WRITE (nout,*) 'The factor D is singular'
       END IF

99999  FORMAT ((5X,1P,4(E11.1,7X)))
    END PROGRAM f07qvfe
