    PROGRAM f08qgfe

!      F08QGF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY: dgemm, dlange => f06raf, dtrsen, nag_wp, x02ajf, &
                              x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: alpha, beta, norm, s, sep
       INTEGER                         :: i, ifail, info, lda, ldc, ldq, ldt,  &
                                          liwork, lwork, m, n
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), c(:,:), q(:,:), t(:,:),      &
                                          wi(:), work(:), wr(:)
       INTEGER, ALLOCATABLE            :: iwork(:)
       LOGICAL, ALLOCATABLE            :: select(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'F08QGF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n
       lda = n
       ldc = n
       ldq = n
       ldt = n
       liwork = (n*n)/4
       lwork = (n*n)/2
       ALLOCATE (a(lda,n),c(ldc,n),q(ldq,n),t(ldt,n),wi(n),work(lwork),wr(n), &
          iwork(liwork),select(n))

!      Read T, Q and the logical array SELECT from data file

       READ (nin,*) (t(i,1:n),i=1,n)
       READ (nin,*) (q(i,1:n),i=1,n)

       READ (nin,*) select(1:n)

!      Compute Q * T * Q**T to find  A
       alpha = 1._nag_wp
       beta = 0._nag_wp
       CALL dgemm('N','N',n,n,n,alpha,q,ldq,t,ldt,beta,c,ldc)
       CALL dgemm('N','T',n,n,n,alpha,c,ldc,q,ldq,beta,a,lda)

!      Print Matrix A, as computed from Q * T * Q**T
!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 0
       CALL x04caf('General',' ',n,n,a,lda,'Matrix A computed from Q*T*Q^T', &
          ifail)

       WRITE (nout,*)
       FLUSH (nout)

!      Reorder the Schur factor T and update the matrix Q to obtain TT and QT

!      The NAG name equivalent of dtrsen is f08qgf
       CALL dtrsen('Both','Vectors',select,n,t,ldt,q,ldq,wr,wi,m,s,sep,work, &
           lwork,iwork,liwork,info)

!      Compute (Q * T * Q^T) - (QT * TT * QT^T) and store in A, 
!      i.e. the difference between reconstructed A using Schur and reordered
!           Schur decompositions.
       alpha = 1._nag_wp
       beta = 0._nag_wp
       CALL dgemm('N','N',n,n,n,alpha,q,ldq,t,ldt,beta,c,ldc)
       alpha = -1._nag_wp
       beta = 1._nag_wp
       CALL dgemm('N','T',n,n,n,alpha,c,ldc,q,ldq,beta,a,lda)

!      Find norm of difference matrix and print warning if it is too large
!      f06raf is the NAG name equivalent of the LAPACK auxiliary dlange
       norm = dlange('O',lda,n,a,lda,work)
       IF (norm>x02ajf()**0.8_nag_wp) THEN
          WRITE (nout,*) 'Norm of A - (QT * TT * QT^T) is much greater than 0.'
          WRITE (nout,*) 'Schur factorization has failed.'
       ELSE
!         Print Result
          WRITE (nout,99999) 'Condition number estimate', &
              ' of the selected cluster of eigenvalues = ', 1.0_nag_wp/s
          WRITE (nout,*)
          WRITE (nout,99999) 'Condition number estimate of the spec', &
              'ified invariant subspace    = ', 1.0_nag_wp/sep
       END IF

99999  FORMAT (1X,A,A,1P,E10.2)

    END PROGRAM f08qgfe
