    PROGRAM f08wnfe

!      F08WNF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp, x02amf, zggev
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nb = 64, nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: small
       INTEGER                         :: i, info, j, lda, ldb, ldvr, lwork, n
!      .. Local Arrays ..
       COMPLEX (KIND=nag_wp), ALLOCATABLE :: a(:,:), alpha(:), b(:,:),         &
                                             beta(:), vr(:,:), work(:)
       COMPLEX (KIND=nag_wp)           :: dummy(1,1)
       REAL (KIND=nag_wp), ALLOCATABLE :: rwork(:)
!      .. Intrinsic Functions ..
       INTRINSIC                          abs, max, nint, real
!      .. Executable Statements ..
       WRITE (nout,*) 'F08WNF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n
       lda = n
       ldb = n
       ldvr = n
       ALLOCATE (a(lda,n),alpha(n),b(ldb,n),beta(n),vr(ldvr,n),rwork(8*n))

!      Use routine workspace query to get optimal workspace.
       lwork = -1
!      The NAG name equivalent of zggev is f08wnf
       CALL zggev('No left vectors','Vectors (right)',n,a,lda,b,ldb,alpha, &
          beta,dummy,1,vr,ldvr,dummy,lwork,rwork,info)

!      Make sure that there is enough workspace for blocksize nb.
       lwork = max((nb+1)*n,nint(real(dummy(1,1))))
       ALLOCATE (work(lwork))

!      Read in the matrices A and B

       READ (nin,*) (a(i,1:n),i=1,n)
       READ (nin,*) (b(i,1:n),i=1,n)

!      Solve the generalized eigenvalue problem

!      The NAG name equivalent of zggev is f08wnf
       CALL zggev('No left vectors','Vectors (right)',n,a,lda,b,ldb,alpha, &
          beta,dummy,1,vr,ldvr,work,lwork,rwork,info)

!      Normalize the eigenvectors
       DO i = 1, n
          vr(1:n,i) = vr(1:n,i)/vr(1,i)
       END DO

       IF (info>0) THEN
          WRITE (nout,*)
          WRITE (nout,99999) 'Failure in ZGGEV. INFO =', info
       ELSE
          small = x02amf()
          DO j = 1, n
             WRITE (nout,*)
             IF ((abs(alpha(j)))*small>=abs(beta(j))) THEN
                WRITE (nout,99998) 'Eigenvalue(', j, ')', &
                   ' is numerically infinite or undetermined', 'ALPHA(', j, &
                   ') = ', alpha(j), ', BETA(', j, ') = ', beta(j)
             ELSE
                WRITE (nout,99997) 'Eigenvalue(', j, ') = ', alpha(j)/beta(j)
             END IF
             WRITE (nout,*)
             WRITE (nout,99996) 'Eigenvector(', j, ')', (vr(i,j),i=1,n)
          END DO

       END IF

99999  FORMAT (1X,A,I4)
99998  FORMAT (1X,A,I2,2A/1X,2(A,I2,A,'(',1P,E11.4,',',1P,E11.4,')'))
99997  FORMAT (1X,A,I2,A,'(',1P,E11.4,',',1P,E11.4,')')
99996  FORMAT (1X,A,I2,A/3(1X,'(',1P,E11.4,',',1P,E11.4,')':))
    END PROGRAM f08wnfe
