    PROGRAM f08xefe

!      F08XEF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dgeqrf, dggbal, dgghrd, dhgeqz, dormqr, nag_wp, &
                               x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, ihi, ilo, info, irows,     &
                                          jwork, lda, ldb, ldq, ldz, lwork, n
       CHARACTER (1)                   :: compq, compz, job
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), alphai(:), alphar(:),        &
                                          b(:,:), beta(:), lscale(:), q(:,:),  &
                                          rscale(:), tau(:), work(:), z(:,:)
!      .. Intrinsic Functions ..
       INTRINSIC                          nint
!      .. Executable Statements ..
       WRITE (nout,*) 'F08XEF Example Program Results'
       FLUSH (nout)

!      Skip heading in data file

       READ (nin,*)
       READ (nin,*) n
       ldq = 1
       ldz = 1
       lda = n
       ldb = n
       lwork = 6*n
       ALLOCATE (alphai(n),alphar(n),beta(n),a(lda,n),lscale(n),q(ldq,ldq), &
          rscale(n),b(ldb,n),tau(n),work(lwork),z(ldz,ldz))

!      READ matrix A from data file
       READ (nin,*) (a(i,1:n),i=1,n)

!      READ matrix B from data file
       READ (nin,*) (b(i,1:n),i=1,n)

!      Balance matrix pair (A,B)
       job = 'B'

!      The NAG name equivalent of dggbal is f08whf
       CALL dggbal(job,n,a,lda,b,ldb,ilo,ihi,lscale,rscale,work,info)

!      Matrix A after balancing

!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 0
       CALL x04caf('General',' ',n,n,a,lda,'Matrix A after balancing',ifail)

       WRITE (nout,*)
       FLUSH (nout)

!      Matrix B after balancing

       ifail = 0
       CALL x04caf('General',' ',n,n,b,ldb,'Matrix B after balancing',ifail)

       WRITE (nout,*)
       FLUSH (nout)

!      Reduce B to triangular form using QR
       irows = ihi + 1 - ilo

!      The NAG name equivalent of dgeqrf is f08aef
       CALL dgeqrf(irows,irows,b(ilo,ilo),ldb,tau,work,lwork,info)

!      Apply the orthogonal transformation to matrix A
!      The NAG name equivalent of dormqr is f08agf
       CALL dormqr('L','T',irows,irows,irows,b(ilo,ilo),ldb,tau,a(ilo,ilo), &
          lda,work,lwork,info)

!      Compute the generalized Hessenberg form of (A,B) -> (H,T)
       compq = 'N'
       compz = 'N'

!      The NAG name equivalent of dgghrd is f08wef
       CALL dgghrd(compq,compz,irows,1,irows,a(ilo,ilo),lda,b(ilo,ilo),ldb,q, &
          ldq,z,ldz,info)

!      Matrix A (H) in generalized Hessenberg form.

       ifail = 0
       CALL x04caf('General',' ',n,n,a,lda,'Matrix A in Hessenberg form', &
          ifail)

       WRITE (nout,*)
       FLUSH (nout)

!      Matrix B (T) in generalized Hessenberg form.

       ifail = 0
       CALL x04caf('General',' ',n,n,b,ldb,'Matrix B is triangular',ifail)

!      Routine DHGEQZ
!      Workspace query: jwork = -1

       jwork = -1
       job = 'E'

!      The NAG name equivalent of dhgeqz is f08xef 
       CALL dhgeqz(job,compq,compz,n,ilo,ihi,a,lda,b,ldb,alphar,alphai,beta,q, &
          ldq,z,ldz,work,jwork,info)

       WRITE (nout,*)
       WRITE (nout,99999) nint(work(1))
       WRITE (nout,99998) lwork
       WRITE (nout,*)

!      Compute the generalized Schur form
!      if the workspace lwork is adequate

       IF (nint(work(1))<=lwork) THEN

!         The NAG name equivalent of dhgeqz is f08xef
          CALL dhgeqz(job,compq,compz,n,ilo,ihi,a,lda,b,ldb,alphar,alphai, &
             beta,q,ldq,z,ldz,work,lwork,info)

!         Print the generalized eigenvalues

          WRITE (nout,99997)

          DO i = 1, n
             IF (beta(i)/=0.0E0_nag_wp) THEN
                WRITE (nout,99996) i, '(', alphar(i)/beta(i), ',', &
                   alphai(i)/beta(i), ')'
             ELSE
                WRITE (nout,99994) i
             END IF
          END DO
       ELSE
          WRITE (nout,99995)
       END IF

99999  FORMAT (1X,'Minimal required LWORK = ',I6)
99998  FORMAT (1X,'Actual value of  LWORK = ',I6)
99997  FORMAT (1X,'Generalized eigenvalues')
99996  FORMAT (1X,I4,5X,A,F7.3,A,F7.3,A)
99995  FORMAT (1X,'Insufficient workspace allocated for call to F08XEF/DHGEQZ' &
          )
99994  FORMAT (1X,I4,'Eigenvalue is infinite')
    END PROGRAM f08xefe
