    PROGRAM f11bdfe

!      F11BDF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : f11bdf, f11bef, f11bff, f11daf, f11dbf, f11xaf, &
                               nag_wp
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: anorm, dtol, sigmax, stplhs, stprhs, &
                                          tol
       INTEGER                         :: i, ifail, ifail1, irevcm, iterm,     &
                                          itn, la, lfill, liwork, lwork,       &
                                          lwreq, m, maxitn, monit, n, nnz,     &
                                          nnzc, npivm
       CHARACTER (8)                   :: method
       CHARACTER (1)                   :: milu, norm, precon, pstrat, weight
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:), b(:), wgt(:), work(:), x(:)
       INTEGER, ALLOCATABLE            :: icol(:), idiag(:), ipivp(:),         &
                                          ipivq(:), irow(:), istr(:), iwork(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'F11BDF Example Program Results'

!      Skip heading in data file

       READ (nin,*)
       READ (nin,*) n
       READ (nin,*) nnz
       la = 3*nnz
       liwork = 7*n + 2
       lwork = 200
       ALLOCATE (a(la),b(n),wgt(n),work(lwork),x(n),icol(la),idiag(n), &
          ipivp(n),ipivq(n),irow(la),istr(n+1),iwork(liwork))

!      Read or initialize the parameters for the iterative solver

       READ (nin,*) method
       READ (nin,*) precon, norm, weight, iterm
       READ (nin,*) m, tol, maxitn
       READ (nin,*) monit
       anorm = 0.0E0_nag_wp
       sigmax = 0.0E0_nag_wp

!      Read the parameters for the preconditioner

       READ (nin,*) lfill, dtol
       READ (nin,*) milu, pstrat

!      Read the non-zero elements of the matrix A

       DO i = 1, nnz
          READ (nin,*) a(i), irow(i), icol(i)
       END DO

!      Read right-hand side vector b and initial approximate solution

       READ (nin,*) b(1:n)
       READ (nin,*) x(1:n)

!      Calculate incomplete LU factorization

       milu = 'N'

!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 0
       CALL f11daf(n,nnz,a,la,irow,icol,lfill,dtol,pstrat,milu,ipivp,ipivq, &
          istr,idiag,nnzc,npivm,iwork,liwork,ifail)

!      Call F11BDF to initialize the solver

       ifail = 0
       CALL f11bdf(method,precon,norm,weight,iterm,n,m,tol,maxitn,anorm, &
          sigmax,monit,lwreq,work,lwork,ifail)

!      Call repeatedly F11BEF to solve the equations
!      Note that the arrays B and X are overwritten

!      On final exit, X will contain the solution and B the
!      residual vector

       irevcm = 0
       lwreq = lwork

       ifail = 1
LOOP:  DO
          CALL f11bef(irevcm,x,b,wgt,work,lwreq,ifail)

          IF (irevcm/=4) THEN
             ifail1 = -1
             SELECT CASE (irevcm)
             CASE (-1)

                CALL f11xaf('Transpose',n,nnz,a,irow,icol,'No checking',x,b, &
                   ifail1)

             CASE (1)

                CALL f11xaf('No transpose',n,nnz,a,irow,icol,'No checking',x, &
                   b,ifail1)

             CASE (2)

                CALL f11dbf('No transpose',n,a,la,irow,icol,ipivp,ipivq,istr, &
                   idiag,'No checking',x,b,ifail1)

             CASE (3)

                ifail1 = 0
                CALL f11bff(itn,stplhs,stprhs,anorm,sigmax,work,lwreq,ifail1)

                WRITE (nout,99999) itn, stplhs
                WRITE (nout,99998)
                WRITE (nout,99997) (x(i),b(i),i=1,n)
             END SELECT
             IF (ifail1/=0) irevcm = 6
          ELSE IF (ifail/=0) THEN
             WRITE (nout,99993) ifail
             GO TO 20
          ELSE
             EXIT LOOP
          END IF
       END DO LOOP

!      Obtain information about the computation

       ifail1 = 0
       CALL f11bff(itn,stplhs,stprhs,anorm,sigmax,work,lwreq,ifail1)

!      Print the output data

       WRITE (nout,99996)
       WRITE (nout,99995) 'Number of iterations for convergence:    ', itn
       WRITE (nout,99994) 'Residual norm:                           ', stplhs
       WRITE (nout,99994) 'Right-hand side of termination criterion:', stprhs
       WRITE (nout,99994) '1-norm of matrix A:                      ', anorm

!      Output x

       WRITE (nout,99998)
       WRITE (nout,99997) (x(i),b(i),i=1,n)
20     CONTINUE

99999  FORMAT (/1X,'Monitoring at iteration no.',I4/1X,1P,'residual no', &
          'rm: ',E14.4)
99998  FORMAT (/2X,'  Solution vector',2X,' Residual vector')
99997  FORMAT (1X,1P,E16.4,1X,E16.4)
99996  FORMAT (/1X,'Final Results')
99995  FORMAT (1X,A,I4)
99994  FORMAT (1X,A,1P,E14.4)
99993  FORMAT (1X/1X,' ** F11BEF returned with IFAIL = ',I5)
    END PROGRAM f11bdfe
