    PROGRAM g05rkfe

!      G05RKF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : g05kff, g05rkf, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: lseed = 1, nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: theta
       INTEGER                         :: genid, ifail, ldx, lstate, m, n,     &
                                          sdx, sorder, subid
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: x(:,:)
       INTEGER                         :: seed(lseed)
       INTEGER, ALLOCATABLE            :: state(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'G05RKF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)

!      Skip heading in data file
       READ (nin,*)

!      Read in the base generator information and seed
       READ (nin,*) genid, subid, seed(1)

!      Initial call to initialiser to get size of STATE array
       lstate = 0
       ALLOCATE (state(lstate))
       ifail = 0
       CALL g05kff(genid,subid,seed,lseed,state,lstate,ifail)

!      Reallocate STATE
       DEALLOCATE (state)
       ALLOCATE (state(lstate))

!      Initialize the generator to a repeatable sequence
       ifail = 0
       CALL g05kff(genid,subid,seed,lseed,state,lstate,ifail)

!      Read in sample size, number of dimensions and order
       READ (nin,*) n, m, sorder

       IF (sorder==1) THEN
!         X(N,M)
          ldx = n
          sdx = m
       ELSE
!         X(M,N)
          ldx = m
          sdx = n
       END IF
       ALLOCATE (x(ldx,sdx))

!      Read in parameter
       READ (nin,*) theta

!      Generate variates
       ifail = 0
       CALL g05rkf(n,m,theta,sorder,state,x,ldx,sdx,ifail)

!      Display the variates
       IF (sorder==1) THEN
!         X(N,M)
          ifail = 0
          CALL x04caf('General',' ',n,m,x,ldx, &
             'Uniform variates with copula joint distribution',ifail)
       ELSE
!         X(M,N)
          ifail = 0
          CALL x04caf('General',' ',m,n,x,ldx, &
             'Uniform variates with copula joint distribution',ifail)
       END IF

    END PROGRAM g05rkfe
